@@ -15,12 +15,11 @@
  *
  *  You should have received a copy of the GNU Library General Public License
  *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- *  Boston, MA 02111-1307, USA.
+ *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ *  Boston, MA 02110-1301, USA.
  **/
 
 #include "kateprinter.h"
-#include "kateprinter.moc"
 
 #include <kateconfig.h>
 #include <katedocument.h>
@@ -42,6 +41,7 @@
 #include <kuser.h> // for loginName
 
 #include <qpainter.h>
+#include <qpopupmenu.h>
 #include <qpaintdevicemetrics.h>
 #include <qcheckbox.h>
 #include <qcombobox.h>
@@ -57,6 +57,7 @@
 //BEGIN KatePrinter
 bool KatePrinter::print (KateDocument *doc)
 {
+#ifndef Q_WS_WIN //TODO: reenable
   KPrinter printer;
 
   // docname is now always there, including the right Untitled name
@@ -244,7 +245,7 @@ bool KatePrinter::print (KateDocument *doc)
            QString footerTags = printer.option("app-kate-footerformat");
            int pos = reTags.search( footerTags );
            QString rep;
-           while ( pos > 0 )
+           while ( pos > -1 )
            {
              rep = tags[reTags.cap( 1 )];
              footerTags.replace( (uint)pos, 2, rep );
@@ -317,7 +318,7 @@ bool KatePrinter::print (KateDocument *doc)
          int _items ( 0 );
          while ( ( _d = it.current()) != 0 )
          {
-           _widest = QMAX( _widest, ((QFontMetrics)(
+           _widest = kMax( _widest, ((QFontMetrics)(
                                 _d->bold() ?
                                   _d->italic() ?
                                     renderer.config()->fontStruct()->myFontMetricsBI :
@@ -500,7 +501,11 @@ bool KatePrinter::print (KateDocument *doc)
            if ( useGuide && currentPage == 1 )
            {  // FIXME - this may span more pages...
              // draw a box unless we have boxes, in which case we end with a box line
-             paint.setPen( renderer.config()->selectionColor() );
+
+             // use color of dsNormal for the title string and the hline
+             KateAttributeList _dsList;
+             KateHlManager::self()->getDefaults ( renderer.config()->schema(), _dsList );
+             paint.setPen( _dsList.at(0)->textColor() );
              int _marg = 0; // this could be available globally!??
              if ( useBox )
              {
@@ -621,12 +626,14 @@ bool KatePrinter::print (KateDocument *doc)
      return true;
   }
 
+#endif //!Q_WS_WIN
   return false;
 }
 //END KatePrinter
 
+#ifndef Q_WS_WIN //TODO: reenable
 //BEGIN KatePrintTextSettings
-KatePrintTextSettings::KatePrintTextSettings( KPrinter */*printer*/, QWidget *parent, const char *name )
+KatePrintTextSettings::KatePrintTextSettings( KPrinter * /*printer*/, QWidget *parent, const char *name )
   : KPrintDialogPage( parent, name )
 {
   setTitle( i18n("Te&xt Settings") );
@@ -687,7 +694,7 @@ void KatePrintTextSettings::enableSelection( bool enable )
 //END KatePrintTextSettings
 
 //BEGIN KatePrintHeaderFooter
-KatePrintHeaderFooter::KatePrintHeaderFooter( KPrinter */*printer*/, QWidget *parent, const char *name )
+KatePrintHeaderFooter::KatePrintHeaderFooter( KPrinter * /*printer*/, QWidget *parent, const char *name )
   : KPrintDialogPage( parent, name )
 {
   setTitle( i18n("Hea&der && Footer") );
@@ -895,7 +902,7 @@ void KatePrintHeaderFooter::setHFFont()
 
 //BEGIN KatePrintLayout
 
-KatePrintLayout::KatePrintLayout( KPrinter */*printer*/, QWidget *parent, const char *name )
+KatePrintLayout::KatePrintLayout( KPrinter * /*printer*/, QWidget *parent, const char *name )
   : KPrintDialogPage( parent, name )
 {
   setTitle( i18n("L&ayout") );
@@ -992,4 +999,7 @@ void KatePrintLayout::setOptions( const QMap<QString,QString>& opts )
 }
 //END KatePrintLayout
 
+#include "kateprinter.moc"
+#endif //!Q_WS_WIN
+
 // kate: space-indent on; indent-width 2; replace-tabs on;
