@@ -13,14 +13,16 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include "kaccel.h"
 
 #include <qaccel.h>
+#include <qguardedptr.h>
 #include <qpopupmenu.h>
+#include <qregexp.h>
 #include <qstring.h>
 #include <qtimer.h>
 
@@ -45,6 +47,36 @@
 //---------------------------------------------------------------------
 // KAccelEventHandler
 //---------------------------------------------------------------------
+//
+// In KAccelEventHandler::x11Event we do our own X11 keyboard event handling
+// This allows us to map the Win key to Qt::MetaButton, Qt does not know about
+// the Win key.
+//
+// KAccelEventHandler::x11Event will generate an AccelOverride event. The
+// AccelOverride event is abused a bit to ensure that KAccelPrivate::eventFilter
+// (as an event filter on the toplevel widget) will get the key event first
+// (in the form of AccelOverride) before any of the intermediate widgets are
+// able to process it.
+//
+// Qt normally sends an AccelOverride, Accel and then a KeyPress event.
+// A widget can accept the AccelOverride event in which case the Accel event will be
+// skipped and the KeyPress is followed immediately.
+// If the Accel event is accepted, no KeyPress event will follow.
+//
+// KAccelEventHandler::x11Event converts a X11 keyboard event into an AccelOverride
+// event, there are now two possibilities:
+//
+// 1) If KAccel intercepts the AccelOverride we are done and can consider the X11
+// keyboard event as handled.
+// 2) If another widget accepts the AccelOverride, it will expect to get a normal
+// Qt generated KeyPress event afterwards. So we let Qt handle the X11 keyboard event
+// again. However, this will first generate an AccelOverride event, and we already
+// had send that one. To compnesate for this, the global event filter in KApplication
+// is instructed to eat the next AccelOveride event. Qt will then send a normal KeyPress
+// event and from then on everything is normal again.
+//
+// kde_g_bKillAccelOverride is used to tell KApplication::notify to eat the next
+// AccelOverride event.
 
 bool kde_g_bKillAccelOverride = false;
 
@@ -75,6 +107,7 @@ KAccelEventHandler* KAccelEventHandler::g_pSelf = 0;
 bool KAccelEventHandler::g_bAccelActivated = false;
 
 KAccelEventHandler::KAccelEventHandler()
+    : QWidget( 0, "KAccelEventHandler" )
 {
 #	ifdef Q_WS_X11
 	if ( kapp )
@@ -136,8 +169,10 @@ KAccelPrivate::KAccelPrivate( KAccel* pParent, QWidget* pWatch )
 	m_bAutoUpdate = true;
 	connect( (QAccel*)m_pAccel, SIGNAL(activated(int)), this, SLOT(slotKeyPressed(int)) );
 
+#ifdef Q_WS_X11 //only makes sense if KAccelEventHandler is working
 	if( m_pWatch )
 		m_pWatch->installEventFilter( this );
+#endif
 	KAccelEventHandler::self();
 }
 
@@ -158,7 +193,7 @@ bool KAccelPrivate::setEnabled( const QString& sAction, bool bEnable )
 
 	pAction->setEnabled( bEnable );
 
-	QMap<int, KAccelAction*>::iterator it = m_mapIDToAction.begin();
+	QMap<int, KAccelAction*>::const_iterator it = m_mapIDToAction.begin();
 	for( ; it != m_mapIDToAction.end(); ++it ) {
 		if( *it == pAction )
 			((QAccel*)m_pAccel)->setItemEnabled( it.key(), bEnable );
@@ -199,7 +234,11 @@ bool KAccelPrivate::connectKey( KAccelAction& action, const KKeyServer::Key& key
 	m_mapIDToKey[nID] = keyQt;
 
 	if( action.objSlotPtr() && action.methodSlotPtr() ) {
+#ifdef Q_WS_WIN /** @todo TEMP: new implementation (commit #424926) didn't work */
 		((QAccel*)m_pAccel)->connectItem( nID, action.objSlotPtr(), action.methodSlotPtr() );
+#else
+		((QAccel*)m_pAccel)->connectItem( nID, this, SLOT(slotKeyPressed(int)));
+#endif
 		if( !action.isEnabled() )
 			((QAccel*)m_pAccel)->setItemEnabled( nID, false );
 	}
@@ -272,7 +311,7 @@ void KAccelPrivate::slotKeyPressed( int id )
 		// This is needed for when there are multiple actions
 		//  with the same shortcut where all but one is disabled.
 		// pMenu->count() also counts the menu title, so one shortcut will give count = 2.
-		if( pMenu->count() == 2 && static_cast<int>( pMenu->accel(1) ) == 0 ) { // ### use the faster isEmpty() once we require Qt 3.1 (Simon)
+		if( pMenu->count() == 2 && pMenu->accel(1).isEmpty() ) {
 			int iAction = pMenu->idAt(1);
 			slotMenuActivated( iAction );
 		} else {
@@ -292,11 +331,15 @@ void KAccelPrivate::slotMenuActivated( int iAction )
 {
 	kdDebug(125) << "KAccelPrivate::slotMenuActivated( " << iAction << " )" << endl;
 	KAccelAction* pAction = actions().actionPtr( iAction );
+#ifdef Q_WS_WIN /** @todo TEMP: new implementation (commit #424926) didn't work */
 	if( pAction ) {
 		connect( this, SIGNAL(menuItemActivated()), pAction->objSlotPtr(), pAction->methodSlotPtr() );
 		emit menuItemActivated();
 		disconnect( this, SIGNAL(menuItemActivated()), pAction->objSlotPtr(), pAction->methodSlotPtr() );
 	}
+#else
+	emitActivatedSignal( pAction );
+#endif
 }
 
 bool KAccelPrivate::eventFilter( QObject* /*pWatched*/, QEvent* pEvent )
@@ -316,9 +359,16 @@ bool KAccelPrivate::eventFilter( QObject* /*pWatched*/, QEvent* pEvent )
 					KAccelAction* pAction = m_mapIDToAction[nID];
 					if( !pAction->isEnabled() )
 						continue;
+#ifdef Q_WS_WIN /** @todo TEMP: new implementation (commit #424926) didn't work */
+					QGuardedPtr<KAccelPrivate> me = this;
 					connect( this, SIGNAL(menuItemActivated()), pAction->objSlotPtr(), pAction->methodSlotPtr() );
 					emit menuItemActivated();
-					disconnect( this, SIGNAL(menuItemActivated()), pAction->objSlotPtr(), pAction->methodSlotPtr() );
+					if (me) {
+						disconnect( me, SIGNAL(menuItemActivated()), pAction->objSlotPtr(), pAction->methodSlotPtr() );
+					}
+#else
+					emitActivatedSignal( pAction );
+#endif
 				} else
 					slotKeyPressed( nID );
 
@@ -331,6 +381,32 @@ bool KAccelPrivate::eventFilter( QObject* /*pWatched*/, QEvent* pEvent )
 	return false;
 }
 
+#ifndef Q_WS_WIN /** @todo TEMP: new implementation (commit #424926) didn't work */
+void KAccelPrivate::emitActivatedSignal( KAccelAction* pAction )
+{
+	if( pAction ) {
+		QGuardedPtr<KAccelPrivate> me = this;
+		QRegExp reg( "([ ]*KAccelAction.*)" );
+		if( reg.search( pAction->methodSlotPtr()) >= 0 ) {
+			connect( this, SIGNAL(menuItemActivated(KAccelAction*)),
+				pAction->objSlotPtr(), pAction->methodSlotPtr() );
+			emit menuItemActivated( pAction );
+			if (me)
+				disconnect( me, SIGNAL(menuItemActivated(KAccelAction*)),
+					pAction->objSlotPtr(), pAction->methodSlotPtr() );
+		} else {
+			connect( this, SIGNAL(menuItemActivated()),
+				pAction->objSlotPtr(), pAction->methodSlotPtr() );
+			emit menuItemActivated();
+			if (me)
+				disconnect( me, SIGNAL(menuItemActivated()),
+					pAction->objSlotPtr(), pAction->methodSlotPtr() );
+
+		}
+	}
+}
+#endif
+
 //---------------------------------------------------------------------
 // KAccel
 //---------------------------------------------------------------------
