@@ -17,8 +17,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <config.h>
@@ -95,21 +95,21 @@ extern "C" {
 # endif
 #endif
 
-#if defined (__FreeBSD__) || defined (__NetBSD__) || defined (__OpenBSD__) || defined (__bsdi__) || defined(__APPLE__)
+#if defined (__FreeBSD__) || defined (__NetBSD__) || defined (__OpenBSD__) || defined (__bsdi__) || defined(__APPLE__) || defined (__DragonFly__)
 # define _tcgetattr(fd, ttmode) ioctl(fd, TIOCGETA, (char *)ttmode)
 #else
-# if defined(_HPUX_SOURCE) || defined(__Lynx__)
+# if defined(_HPUX_SOURCE) || defined(__Lynx__) || defined (__CYGWIN__)
 #  define _tcgetattr(fd, ttmode) tcgetattr(fd, ttmode)
 # else
 #  define _tcgetattr(fd, ttmode) ioctl(fd, TCGETS, (char *)ttmode)
 # endif
 #endif
 
-#if defined (__FreeBSD__) || defined (__NetBSD__) || defined (__OpenBSD__) || defined (__bsdi__) || defined(__APPLE__)
+#if defined (__FreeBSD__) || defined (__NetBSD__) || defined (__OpenBSD__) || defined (__bsdi__) || defined(__APPLE__) || defined (__DragonFly__)
 # define _tcsetattr(fd, ttmode) ioctl(fd, TIOCSETA, (char *)ttmode)
 #else
-# ifdef _HPUX_SOURCE
-#  define _tcsetattr(fd, ttmode) tcsetattr(fd, TCSANOW, ttmode)
+# if defined(_HPUX_SOURCE) || defined(__CYGWIN__)
+#  define _tcsetattr(fd, ttmode) tcsetattr(fd, TCSANOW, ttmode) 
 # else
 #  define _tcsetattr(fd, ttmode) ioctl(fd, TCSETS, (char *)ttmode)
 # endif
@@ -164,6 +164,7 @@ public:
 struct KPtyPrivate {
    KPtyPrivate() :
      xonXoff(false),
+     utf8(false),
      masterFd(-1), slaveFd(-1)
    {
      memset(&winSize, 0, sizeof(winSize));
@@ -172,6 +173,7 @@ struct KPtyPrivate {
    }
 
    bool xonXoff : 1;
+   bool utf8    : 1;
    int masterFd;
    int slaveFd;
    struct winsize winSize;
@@ -277,7 +279,9 @@ bool KPty::open()
  gotpty:
   struct stat st;
   if (stat(d->ttyName.data(), &st))
-    return false; // this just cannot happen ... *cough*
+    return false; // this just cannot happen ... *cough*  Yeah right, I just
+                  // had it happen when pty #349 was allocated.  I guess
+                  // there was some sort of leak?  I only had a few open.
   if (((st.st_uid != getuid()) ||
        (st.st_mode & (S_IRGRP|S_IXGRP|S_IROTH|S_IWOTH|S_IXOTH))) &&
       !chownpty(true))
@@ -323,6 +327,13 @@ bool KPty::open()
   else
     ttmode.c_iflag |= (IXOFF | IXON);
 
+#ifdef IUTF8
+  if (!d->utf8)
+    ttmode.c_iflag &= ~IUTF8;
+  else
+    ttmode.c_iflag |= IUTF8;
+#endif
+
   ttmode.c_cc[VINTR] = CTRL('C' - '@');
   ttmode.c_cc[VQUIT] = CTRL('\\' - '@');
   ttmode.c_cc[VERASE] = 0177;
@@ -349,7 +360,7 @@ void KPty::close()
          if (!stat(d->ttyName.data(), &st)) {
             chown(d->ttyName.data(), 0, st.st_gid == getgid() ? 0 : -1);
             chmod(d->ttyName.data(), S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
-	 }
+         }
       } else {
          fcntl(d->masterFd, F_SETFD, 0);
          chownpty(false);
@@ -478,6 +489,28 @@ void KPty::setXonXoff(bool useXonXoff)
   }
 }
 
+void KPty::setUtf8Mode(bool useUtf8)
+{
+  d->utf8 = useUtf8;
+#ifdef IUTF8
+  if (d->masterFd >= 0) {
+    // without the '::' some version of HP-UX thinks, this declares
+    // the struct in this class, in this method, and fails to find
+    // the correct tc[gs]etattr
+    struct ::termios ttmode;
+
+    _tcgetattr(d->masterFd, &ttmode);
+
+    if (!useUtf8)
+      ttmode.c_iflag &= ~IUTF8;
+    else
+      ttmode.c_iflag |= IUTF8;
+
+    _tcsetattr(d->masterFd, &ttmode);
+  }
+#endif
+}
+
 const char *KPty::ttyName() const
 {
     return d->ttyName.data();
