@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: main.cpp,v 1.3 2001/03/30 21:48:48 waba Exp $
+** $Id: main.cpp 117166 2001-10-10 17:40:42Z mueller $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -9,7 +9,7 @@
 *****************************************************************************/
 
 #include "helpwindow.h"
-#include <kapp.h>
+#include <kapplication.h>
 #include <qwindowsstyle.h>
 #include <qstylesheet.h>
 #include <stdlib.h>
