@@ -18,8 +18,8 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 #ifndef _CSS_BASE_H
@@ -48,12 +48,33 @@ namespace DOM {
 
     class DocumentImpl;
 
+    struct CSSNamespace {
+        DOMString m_prefix;
+        DOMString m_uri;
+        CSSNamespace* m_parent;
+
+        CSSNamespace(const DOMString& p, const DOMString& u, CSSNamespace* parent)
+            :m_prefix(p), m_uri(u), m_parent(parent) {}
+        ~CSSNamespace() { delete m_parent; }
+
+        const DOMString& uri() { return m_uri; }
+        const DOMString& prefix() { return m_prefix; }
+
+        CSSNamespace* namespaceForPrefix(const DOMString& prefix) {
+            if (prefix == m_prefix)
+                return this;
+            if (m_parent)
+                return m_parent->namespaceForPrefix(prefix);
+            return 0;
+        }
+    };
+
 // this class represents a selector for a StyleRule
     class CSSSelector
     {
     public:
 	CSSSelector()
-	    : tagHistory(0), simpleSelector(0), attr(0), tag(0xffff), relation( Descendant ),
+	    : tagHistory(0), simpleSelector(0), attr(0), tag(anyQName), relation( Descendant ),
 	      match( None ), nonCSSHint( false ), pseudoId( 0 ), _pseudoType(PseudoNotParsed)
         {}
 
@@ -86,9 +107,11 @@ namespace DOM {
 	    Id,
 	    Exact,
 	    Set,
+	    Class,
 	    List,
 	    Hyphen,
-	    Pseudo,
+	    PseudoClass,
+	    PseudoElement,
 	    Contain,   // css3: E[foo*="bar"]
 	    Begin,     // css3: E[foo^="bar"]
 	    End        // css3: E[foo$="bar"]
@@ -118,25 +141,30 @@ namespace DOM {
             PseudoNthOfType,
             PseudoNthLastOfType,
             PseudoOnlyOfType,
-            PseudoFirstLine,
-	    PseudoFirstLetter,
 	    PseudoLink,
 	    PseudoVisited,
 	    PseudoHover,
 	    PseudoFocus,
 	    PseudoActive,
             PseudoTarget,
-	    PseudoBefore,
-	    PseudoAfter,
             PseudoLang,
             PseudoNot,
             PseudoContains,
             PseudoRoot,
-            PseudoSelection,
             PseudoEnabled,
             PseudoDisabled,
             PseudoChecked,
-            PseudoIndeterminate
+            PseudoIndeterminate,
+// pseudo-elements:
+    // inherited:
+            PseudoFirstLine,
+            PseudoFirstLetter,
+            PseudoSelection,
+    // generated:
+            PseudoBefore,
+            PseudoAfter,
+            PseudoMarker,
+            PseudoReplaced
 	};
 
 	PseudoType pseudoType() const {
@@ -153,10 +181,10 @@ namespace DOM {
         DOM::NodeImpl::Id tag;
 
 	Relation relation     : 3;
-	Match 	 match         : 4;
+	mutable Match 	 match         : 4;
 	bool	nonCSSHint : 1;
-	unsigned int pseudoId : 3;
-	mutable PseudoType _pseudoType : 5;
+	unsigned int pseudoId : 4;
+	mutable PseudoType _pseudoType : 6;
 
     private:
 	void extractPseudoType() const;
