@@ -18,8 +18,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 
     This class provides all functionality needed for loading images, style sheets and html
     pages from the web. It has a memory cache for these objects.
@@ -68,10 +68,18 @@
 #include "khtml_factory.h"
 #include "khtml_part.h"
 
+#ifdef IMAGE_TITLES
+#include <qfile.h>
+#include <kfilemetainfo.h>
+#include <ktempfile.h>
+#endif
+
 #include "html/html_documentimpl.h"
 #include "css/css_stylesheetimpl.h"
 #include "xml/dom_docimpl.h"
 
+#include "blocked_icon.cpp"
+
 using namespace khtml;
 using namespace DOM;
 
@@ -86,6 +94,10 @@ struct LRUList {
 static LRUList m_LRULists[MAX_LRU_LISTS];
 static LRUList* getLRUListFor(CachedObject* o);
 
+CachedObjectClient::~CachedObjectClient()
+{
+}
+
 CachedObject::~CachedObject()
 {
     Cache::removeFromLRUList(this);
@@ -170,6 +182,7 @@ QTextCodec* CachedObject::codecForBuffer( const QString& charset, const QByteArr
     uchar* d = ( uchar* ) buffer.data();
     int s = buffer.size();
 
+    // BOM
     if ( s >= 3 &&
          d[0] == 0xef && d[1] == 0xbb && d[2] == 0xbf)
          return QTextCodec::codecForMib( 106 ); // UTF-8
@@ -178,6 +191,7 @@ QTextCodec* CachedObject::codecForBuffer( const QString& charset, const QByteArr
                     (d[0] == 0xfe && d[1] == 0xff)))
         return QTextCodec::codecForMib( 1000 ); // UCS-2
 
+    // Link or @charset
     if(!charset.isEmpty())
     {
 	QTextCodec* c = KGlobal::charsets()->codecForName(charset);
@@ -188,7 +202,8 @@ QTextCodec* CachedObject::codecForBuffer( const QString& charset, const QByteArr
         return c;
     }
 
-    return QTextCodec::codecForMib(4); // latin-1
+    // Default
+    return QTextCodec::codecForMib( 4 ); // latin 1
 }
 
 // -------------------------------------------------------------------------------------------
@@ -204,6 +219,7 @@ CachedCSSStyleSheet::CachedCSSStyleSheet(DocLoader* dl, const DOMString &url, KI
     ah += "*/*;q=0.1";
     setAccept( ah );
     m_hadError = false;
+    m_wasBlocked = false;
     m_err = 0;
     // load the file
     Cache::loader()->load(dl, this, false);
@@ -227,7 +243,7 @@ void CachedCSSStyleSheet::ref(CachedObjectClient *c)
 	if (m_hadError)
 	    c->error( m_err, m_errText );
 	else
-	    c->setStyleSheet( m_url, m_sheet );
+	    c->setStyleSheet( m_url, m_sheet, m_charset );
     }
 }
 
@@ -237,10 +253,19 @@ void CachedCSSStyleSheet::data( QBuffer &buffer, bool eof )
     buffer.close();
     setSize(buffer.buffer().size());
 
-    QTextCodec* c = codecForBuffer( m_charset, buffer.buffer() );
+//     QString charset = checkCharset( buffer.buffer() );
+    QTextCodec* c = 0;
+    if (!m_charset.isEmpty()) {
+        c = KGlobal::charsets()->codecForName(m_charset);
+        if(c->mibEnum() == 11)  c = QTextCodec::codecForName("iso8859-8-i");
+    }
+    else {
+        c = codecForBuffer( m_charsetHint, buffer.buffer() );
+        m_charset = c->name();
+    }
     QString data = c->toUnicode( buffer.buffer().data(), m_size );
     // workaround Qt bugs
-    m_sheet = data[0] == QChar::byteOrderMark ? DOMString(data.mid( 1 ) ) : DOMString(data);
+    m_sheet = static_cast<QChar>(data[0]) == QChar::byteOrderMark ? DOMString(data.mid( 1 ) ) : DOMString(data);
     m_loading = false;
 
     checkNotify();
@@ -255,7 +280,7 @@ void CachedCSSStyleSheet::checkNotify()
     // it() first increments, then returnes the current item.
     // this avoids skipping an item when setStyleSheet deletes the "current" one.
     for (QPtrDictIterator<CachedObjectClient> it( m_clients ); it.current();)
-        it()->setStyleSheet( m_url, m_sheet );
+        it()->setStyleSheet( m_url, m_sheet, m_charset );
 }
 
 
@@ -272,6 +297,28 @@ void CachedCSSStyleSheet::error( int err, const char* text )
         it()->error( m_err, m_errText );
 }
 
+#if 0
+QString CachedCSSStyleSheet::checkCharset(const QByteArray& buffer ) const
+{
+    int s = buffer.size();
+    if (s <= 12) return m_charset;
+
+    // @charset has to be first or directly after BOM.
+    // CSS 2.1 says @charset should win over BOM, but since more browsers support BOM
+    // than @charset, we default to that.
+    const char* d = (const char*) buffer.data();
+    if (strncmp(d, "@charset \"",10) == 0)
+    {
+        // the string until "; is the charset name
+        char *p = strchr(d+10, '"');
+        if (p == 0) return m_charset;
+        QString charset = QString::fromAscii(d+10, p-(d+10));
+	return charset;
+    }
+    return m_charset;
+}
+#endif
+
 // -------------------------------------------------------------------------------------------
 
 CachedScript::CachedScript(DocLoader* dl, const DOMString &url, KIO::CacheControl _cachePolicy, const char*)
@@ -309,7 +356,7 @@ void CachedScript::data( QBuffer &buffer, bool eof )
 
     QTextCodec* c = codecForBuffer( m_charset, buffer.buffer() );
     QString data = c->toUnicode( buffer.buffer().data(), m_size );
-    m_script = data[0] == QChar::byteOrderMark ? DOMString(data.mid( 1 ) ) : DOMString(data);
+    m_script = static_cast<QChar>(data[0]) == QChar::byteOrderMark ? DOMString(data.mid( 1 ) ) : DOMString(data);
     m_loading = false;
     checkNotify();
 }
@@ -428,6 +475,7 @@ CachedImage::CachedImage(DocLoader* dl, const DOMString &url, KIO::CacheControl
     p = 0;
     pixPart = 0;
     bg = 0;
+    scaled = 0;
     bgColor = qRgba( 0, 0, 0, 0xFF );
     typeChecked = false;
     isFullyTransparent = false;
@@ -437,6 +485,11 @@ CachedImage::CachedImage(DocLoader* dl, const DOMString &url, KIO::CacheControl
     imgSource = 0;
     setAccept( acceptHeader );
     m_showAnimations = dl->showAnimations();
+
+    if ( KHTMLFactory::defaultHTMLSettings()->isAdFiltered( url.string() ) ) {
+        m_wasBlocked = true;
+        CachedObject::finish();
+    }
 }
 
 CachedImage::~CachedImage()
@@ -471,11 +524,19 @@ void CachedImage::deref( CachedObjectClient *c )
 #define BGMINWIDTH      32
 #define BGMINHEIGHT     32
 
-
-const QPixmap &CachedImage::tiled_pixmap(const QColor& newc)
+const QPixmap &CachedImage::tiled_pixmap(const QColor& newc, int xWidth, int xHeight)
 {
-    static QRgb bgTransparant = qRgba( 0, 0, 0, 0xFF );
-    if ( (bgColor != bgTransparant) && (bgColor != newc.rgb()) ) {
+    static QRgb bgTransparent = qRgba( 0, 0, 0, 0xFF );
+
+    QSize s(pixmap_size());
+    int w = xWidth;
+    int h = xHeight;
+    if (w == -1) xWidth = w = s.width();
+    if (h == -1) xHeight = h = s.height();
+
+    if ( ( (bgColor != bgTransparent) && (bgColor != newc.rgb()) ) ||
+         ( bgSize != QSize(xWidth, xHeight)) )
+    {
         delete bg; bg = 0;
     }
 
@@ -487,101 +548,97 @@ const QPixmap &CachedImage::tiled_pixmap(const QColor& newc)
     if (r.isNull()) return r;
 
     // no error indication for background images
-    if(m_hadError) return *Cache::nullPixmap;
+    if(m_hadError||m_wasBlocked) return *Cache::nullPixmap;
 
     bool isvalid = newc.isValid();
-    QSize s(pixmap_size());
-    int w = r.width();
-    int h = r.height();
+
+    const QPixmap* src; //source for pretiling, if any
+
+    //See whether we should scale
+    if (xWidth != s.width() || xHeight != s.height()) {
+        src = &scaled_pixmap(xWidth, xHeight);
+    } else {
+        src = &r;
+    }
+
+    bgSize = QSize(xWidth, xHeight);
+
+    //See whether we can - and should - pre-blend
+    if (isvalid && (r.hasAlphaChannel() || r.mask() )) {
+        bg = new QPixmap(xWidth, xHeight, r.depth());
+        bg->fill(newc);
+        bitBlt(bg, 0, 0, src);
+        bgColor = newc.rgb();
+        src     = bg;
+    } else {
+        bgColor = bgTransparent;
+    }
+
+    //See whether to pre-tile.
     if ( w*h < 8192 )
     {
         if ( r.width() < BGMINWIDTH )
-            w = ((BGMINWIDTH  / s.width())+1) * s.width();
+            w = ((BGMINWIDTH-1) / xWidth + 1) * xWidth;
         if ( r.height() < BGMINHEIGHT )
-            h = ((BGMINHEIGHT / s.height())+1) * s.height();
+            h = ((BGMINHEIGHT-1) / xHeight + 1) * xHeight;
     }
-
-#ifdef Q_WS_X11
-    if ( r.hasAlphaChannel() && 
-         ((w != r.width()) || (h != r.height())) )
+    if ( w != xWidth  || h != xHeight )
     {
-        bg = new QPixmap(w, h);
+//         kdDebug() << "pre-tiling " << s.width() << "," << s.height() << " to " << w << "," << h << endl;
+        QPixmap* oldbg = bg;
+        bg = new QPixmap(w, h, r.depth());
+
         //Tile horizontally on the first stripe
-        for (int x = 0; x < w; x += r.width())
-            copyBlt(bg, x, 0, &r, 0, 0, r.width(), r.height());
-        
+        for (int x = 0; x < w; x += xWidth)
+            copyBlt(bg, x, 0, src, 0, 0, xWidth, xHeight);
+
         //Copy first stripe down
-        for (int y = r.height(); y < h; y += r.height())
-            copyBlt(bg, 0, y, bg, 0, 0, w, r.height());
-        
-        return *bg;
-    }
-#endif    
-    
-    if ( 
-#ifdef Q_WS_X11    
-        !r.hasAlphaChannel() &&
-#endif
-        ( (w != r.width()) || (h != r.height()) || (isvalid && r.mask())) )
-    {
-        QPixmap pix = r;
-        if ( w != r.width() || (isvalid && pix.mask()))
-        {
-            bg = new QPixmap(w, r.height());
-            QPainter p(bg);
-            if(isvalid) p.fillRect(0, 0, w, r.height(), newc);
-            p.drawTiledPixmap(0, 0, w, r.height(), pix);
-            p.end();
+        for (int y = xHeight; y < h; y += xHeight)
+            copyBlt(bg, 0, y, bg, 0, 0, w, xHeight);
 
-            if(!isvalid && pix.mask())
-            {
-                // unfortunately our anti-transparency trick doesn't work here
-                // we need to create a mask.
-                QBitmap newmask(w, r.height());
-                QPainter pm(&newmask);
-                pm.drawTiledPixmap(0, 0, w, r.height(), *pix.mask());
-                bg->setMask(newmask);
-                bgColor = bgTransparant;
-            }
-            else
-                bgColor= newc.rgb();
-            pix = *bg;
-        }
-        if ( h != r.height() )
-        {
-            delete bg;
-            bg = new QPixmap(w, h);
-            QPainter p(bg);
-            if(isvalid) p.fillRect(0, 0, w, h, newc);
-            p.drawTiledPixmap(0, 0, w, h, pix);
-            if(!isvalid && pix.mask())
-            {
-                // unfortunately our anti-transparency trick doesn't work here
-                // we need to create a mask.
-                QBitmap newmask(w, h);
-                QPainter pm(&newmask);
-                pm.drawTiledPixmap(0, 0, w, h, *pix.mask());
-                bg->setMask(newmask);
-                bgColor = bgTransparant;
-            }
-            else
-                bgColor= newc.rgb();
-        }
-        return *bg;
+        if ( src == oldbg )
+            delete oldbg;
     }
 
-    return r;
+    if (bg)
+        return *bg;
+
+    return *src;
 }
 
+const QPixmap &CachedImage::scaled_pixmap( int xWidth, int xHeight )
+{
+    if (scaled) {
+        if (scaled->width() == xWidth && scaled->height() == xHeight)
+            return *scaled;
+        delete scaled;
+    }
+    const QPixmap &r = pixmap();
+    if (r.isNull()) return r;
+
+//     kdDebug() << "scaling " << r.width() << "," << r.height() << " to " << xWidth << "," << xHeight << endl;
+
+    QImage image = r.convertToImage().smoothScale(xWidth, xHeight);
+
+    scaled = new QPixmap(xWidth, xHeight, r.depth());
+    scaled->convertFromImage(image);
+
+    return *scaled;
+}
+
+
 const QPixmap &CachedImage::pixmap( ) const
 {
     if(m_hadError)
         return *Cache::brokenPixmap;
 
+    if(m_wasBlocked)
+        return *Cache::blockedPixmap;
+
     if(m)
     {
         if(m->framePixmap().size() != m->getValidRect().size())
-        {   
+        {
             // pixmap is not yet completely loaded, so we
             // return a clipped version. asserting here
             // that the valid rect is always from 0/0 to fullwidth/ someheight
@@ -606,12 +663,14 @@ const QPixmap &CachedImage::pixmap( ) const
 
 QSize CachedImage::pixmap_size() const
 {
+    if (m_wasBlocked) return Cache::blockedPixmap->size();
     return (m_hadError ? Cache::brokenPixmap->size() : m ? m->framePixmap().size() : ( p ? p->size() : QSize()));
 }
 
 
 QRect CachedImage::valid_rect() const
 {
+    if (m_wasBlocked) return Cache::blockedPixmap->rect();
     return (m_hadError ? Cache::brokenPixmap->rect() : m ? m->getValidRect() : ( p ? p->rect() : QRect()) );
 }
 
@@ -699,6 +758,7 @@ void CachedImage::movieStatus(int status)
         }
         for (QPtrDictIterator<CachedObjectClient> it( m_clients ); it.current();)
             it()->notifyFinished( this );
+	m_status = Cached; //all done
     }
 
 #if 0
@@ -734,6 +794,17 @@ void CachedImage::setShowAnimations( KHTMLSettings::KAnimationAdvice showAnimati
     }
 }
 
+void CachedImage::pauseAnimations()
+{
+    if ( m ) m->pause();
+}
+
+void CachedImage::resumeAnimations()
+{
+    if ( m ) m->unpause();
+}
+
+
 void CachedImage::deleteMovie()
 {
     delete m; m = 0;
@@ -744,7 +815,9 @@ void CachedImage::clear()
     delete m;   m = 0;
     delete p;   p = 0;
     delete bg;  bg = 0;
+    delete scaled;  scaled = 0;
     bgColor = qRgba( 0, 0, 0, 0xff );
+    bgSize = QSize(-1,-1);
     delete pixPart; pixPart = 0;
 
     formatType = 0;
@@ -763,11 +836,12 @@ void CachedImage::data ( QBuffer &_buffer, bool eof )
     if ( !typeChecked )
     {
         // don't attempt incremental loading if we have all the data already
-        assert(!eof);
-
-        formatType = QImageDecoder::formatName( (const uchar*)_buffer.buffer().data(), _buffer.size());
-        if ( formatType && strcmp( formatType, "PNG" ) == 0 )
-            formatType = 0; // Some png files contain multiple images, we want to show only the first one
+        if (!eof)
+        {
+            formatType = QImageDecoder::formatName( (const uchar*)_buffer.buffer().data(), _buffer.size());
+            if ( formatType && strcmp( formatType, "PNG" ) == 0 )
+                formatType = 0; // Some png files contain multiple images, we want to show only the first one
+        }
 
         typeChecked = true;
 
@@ -797,7 +871,16 @@ void CachedImage::data ( QBuffer &_buffer, bool eof )
 #ifdef CACHE_DEBUG
             kdDebug(6060) << "CachedImage::data(): reloading as pixmap:" << endl;
 #endif
-            p = new QPixmap( _buffer.buffer() );
+            p = new QPixmap;
+            {
+            	QBuffer buffer(_buffer.buffer());
+            	buffer.open(IO_ReadOnly);
+                QImageIO io( &buffer, 0 );
+                io.setGamma(2.2); // hardcoded "reasonable value"
+                bool result = io.read();
+                if (result) p->convertFromImage(io.image(), 0);
+            }
+
             // set size of image.
 #ifdef CACHE_DEBUG
             kdDebug(6060) << "CachedImage::data(): image is null: " << p->isNull() << endl;
@@ -812,6 +895,7 @@ void CachedImage::data ( QBuffer &_buffer, bool eof )
 
             for (QPtrDictIterator<CachedObjectClient> it( m_clients ); it.current();)
                 it()->notifyFinished( this );
+            m_status = Cached; //all done
         }
     }
 }
@@ -897,37 +981,37 @@ void DocLoader::setExpireDate(time_t _expireDate, bool relative)
 
 void DocLoader::insertCachedObject( CachedObject* o ) const
 {
+    if ( m_docObjects.find(o) )
+        return;
     m_docObjects.insert( o, o );
     if ( m_docObjects.count() > 3 * m_docObjects.size() )
         m_docObjects.resize(khtml::nextSeed( m_docObjects.size() ) );
 }
 
-bool DocLoader::needReload(const KURL &fullURL)
+bool DocLoader::needReload(CachedObject *existing, const QString& fullURL)
 {
     bool reload = false;
     if (m_cachePolicy == KIO::CC_Verify)
     {
-       if (!m_reloadedURLs.contains(fullURL.url()))
+       if (!m_reloadedURLs.contains(fullURL))
        {
-          CachedObject *existing = Cache::cache->find(fullURL.url());
           if (existing && existing->isExpired())
           {
              Cache::removeCacheEntry(existing);
-             m_reloadedURLs.append(fullURL.url());
+             m_reloadedURLs.append(fullURL);
              reload = true;
           }
        }
     }
     else if ((m_cachePolicy == KIO::CC_Reload) || (m_cachePolicy == KIO::CC_Refresh))
     {
-       if (!m_reloadedURLs.contains(fullURL.url()))
+       if (!m_reloadedURLs.contains(fullURL))
        {
-          CachedObject *existing = Cache::cache->find(fullURL.url());
           if (existing)
           {
              Cache::removeCacheEntry(existing);
           }
-          m_reloadedURLs.append(fullURL.url());
+          m_reloadedURLs.append(fullURL);
           reload = true;
        }
     }
@@ -959,8 +1043,8 @@ CachedCSSStyleSheet *DocLoader::requestStyleSheet( const DOM::DOMString &url, co
     DOCLOADER_SECCHECK(!userSheet);
 
     CachedCSSStyleSheet* s = Cache::requestObject<CachedCSSStyleSheet, CachedObject::CSSStyleSheet>( this, fullURL, accept );
-    if ( s ) {
-        s->setCharset( charset );
+    if ( s && !charset.isEmpty() ) {
+        s->setCharsetHint( charset );
     }
     return s;
 }
@@ -968,9 +1052,12 @@ CachedCSSStyleSheet *DocLoader::requestStyleSheet( const DOM::DOMString &url, co
 CachedScript *DocLoader::requestScript( const DOM::DOMString &url, const QString& charset)
 {
     DOCLOADER_SECCHECK(true);
+    if ( ! KHTMLFactory::defaultHTMLSettings()->isJavaScriptEnabled(fullURL.host()) ||
+           KHTMLFactory::defaultHTMLSettings()->isAdFiltered(fullURL.url()))
+	return 0L;
 
     CachedScript* s = Cache::requestObject<CachedScript, CachedObject::Script>( this, fullURL, 0 );
-    if ( s )
+    if ( s && !charset.isEmpty() )
         s->setCharset( charset );
     return s;
 }
@@ -1013,6 +1100,28 @@ void DocLoader::setShowAnimations( KHTMLSettings::KAnimationAdvice showAnimation
         }
 }
 
+void DocLoader::pauseAnimations()
+{
+    for ( QPtrDictIterator<CachedObject> it( m_docObjects ); it.current(); ++it )
+        if ( it.current()->type() == CachedObject::Image )
+        {
+            CachedImage *img = const_cast<CachedImage*>( static_cast<const CachedImage *>( it.current() ) );
+
+            img->pauseAnimations();
+        }
+}
+
+void DocLoader::resumeAnimations()
+{
+    for ( QPtrDictIterator<CachedObject> it( m_docObjects ); it.current(); ++it )
+        if ( it.current()->type() == CachedObject::Image )
+        {
+            CachedImage *img = const_cast<CachedImage*>( static_cast<const CachedImage *>( it.current() ) );
+
+            img->resumeAnimations();
+        }
+}
+
 // ------------------------------------------------------------------------------------------
 
 Loader::Loader() : QObject()
@@ -1091,6 +1200,8 @@ void Loader::slotFinished( KIO::Job* job )
   }
   else
   {
+      QString cs = j->queryMetaData("charset");
+      if (!cs.isEmpty()) r->object->setCharset(cs);
       r->object->data(r->m_buffer, true);
       emit requestDone( r->m_docLoader, r->object );
       time_t expireDate = j->queryMetaData("expire-date").toLong();
@@ -1098,10 +1209,31 @@ void Loader::slotFinished( KIO::Job* job )
       kdDebug(6060) << "Loader::slotFinished, url = " << j->url().url() << endl;
 #endif
       r->object->setExpireDate( expireDate );
-  }
 
-  if ( r->object->type() == CachedObject::Image )
-      static_cast<CachedImage*>( r->object )->setSuggestedFilename( j->queryMetaData( "content-disposition" ) );
+      if ( r->object->type() == CachedObject::Image ) {
+          QString fn = j->queryMetaData("content-disposition");
+          static_cast<CachedImage*>( r->object )->setSuggestedFilename(fn);
+#ifdef IMAGE_TITLES
+          static_cast<CachedImage*>( r->object )->setSuggestedTitle(fn);
+          KTempFile tf;
+          tf.setAutoDelete(true);
+          tf.file()->writeBlock((const char*)r->m_buffer.buffer().data(), r->m_buffer.size());
+          tf.sync();
+          KFileMetaInfo kfmi(tf.name());
+          if (!kfmi.isEmpty()) {
+              KFileMetaInfoItem i = kfmi.item("Name");
+              if (i.isValid()) {
+                  static_cast<CachedImage*>(r->object)->setSuggestedTitle(i.string());
+              } else {
+                  i = kfmi.item("Title");
+                  if (i.isValid()) {
+                      static_cast<CachedImage*>(r->object)->setSuggestedTitle(i.string());
+                  }
+              }
+          }
+#endif
+      }
+  }
 
   r->object->finish();
 
@@ -1200,16 +1332,17 @@ KIO::Job *Loader::jobForRequest( const DOM::DOMString &url ) const
 // ----------------------------------------------------------------------------
 
 
-QDict<CachedObject> *Cache::cache;
-QPtrList<DocLoader>* Cache::docloader;
-QPtrList<CachedObject> *Cache::freeList;
-Loader *Cache::m_loader;
+QDict<CachedObject> *Cache::cache = 0;
+QPtrList<DocLoader>* Cache::docloader = 0;
+QPtrList<CachedObject> *Cache::freeList = 0;
+Loader *Cache::m_loader = 0;
 
 int Cache::maxSize = DEFCACHESIZE;
 int Cache::totalSizeOfLRU;
 
-QPixmap *Cache::nullPixmap;
-QPixmap *Cache::brokenPixmap;
+QPixmap *Cache::nullPixmap = 0;
+QPixmap *Cache::brokenPixmap = 0;
+QPixmap *Cache::blockedPixmap = 0;
 
 void Cache::init()
 {
@@ -1225,6 +1358,11 @@ void Cache::init()
     if ( !brokenPixmap )
         brokenPixmap = new QPixmap(KHTMLFactory::instance()->iconLoader()->loadIcon("file_broken", KIcon::Desktop, 16, KIcon::DisabledState));
 
+    if ( !blockedPixmap ) {
+        blockedPixmap = new QPixmap();
+        blockedPixmap->loadFromData(blocked_icon_data, blocked_icon_len);
+    }
+
     if ( !m_loader )
         m_loader = new Loader();
 
@@ -1244,15 +1382,40 @@ void Cache::clear()
     cache->setAutoDelete( true );
 
 #ifndef NDEBUG
-    for (QDictIterator<CachedObject> it(*cache); it.current(); ++it)
-        assert(it.current()->canDelete());
-    for (QPtrListIterator<CachedObject> it(*freeList); it.current(); ++it)
-        assert(it.current()->canDelete());
+    bool crash = false;
+    for (QDictIterator<CachedObject> it(*cache); it.current(); ++it) {
+        if (!it.current()->canDelete()) {
+            kdDebug( 6060 ) << " Object in cache still linked to" << endl;
+            kdDebug( 6060 ) << " -> URL: " << it.current()->url() << endl;
+            kdDebug( 6060 ) << " -> #clients: " << it.current()->count() << endl;
+            crash = true;
+//         assert(it.current()->canDelete());
+        }
+    }
+    for (freeList->first(); freeList->current(); freeList->next()) {
+        if (!freeList->current()->canDelete()) {
+            kdDebug( 6060 ) << " Object in freelist still linked to" << endl;
+            kdDebug( 6060 ) << " -> URL: " << freeList->current()->url() << endl;
+            kdDebug( 6060 ) << " -> #clients: " << freeList->current()->count() << endl;
+            crash = true;
+            /*
+            QPtrDictIterator<CachedObjectClient> it(freeList->current()->m_clients);
+            for(;it.current(); ++it) {
+                if (dynamic_cast<RenderObject*>(it.current())) {
+                    kdDebug( 6060 ) << " --> RenderObject" << endl;
+                } else
+                    kdDebug( 6060 ) << " --> Something else" << endl;
+            }*/
+        }
+//         assert(freeList->current()->canDelete());
+    }
+    assert(!crash);
 #endif
 
     delete cache; cache = 0;
     delete nullPixmap; nullPixmap = 0;
     delete brokenPixmap; brokenPixmap = 0;
+    delete blockedPixmap; blockedPixmap = 0;
     delete m_loader;  m_loader = 0;
     delete docloader; docloader = 0;
     delete freeList; freeList = 0;
@@ -1263,16 +1426,17 @@ CachedObjectType* Cache::requestObject( DocLoader* dl, const KURL& kurl, const c
 {
     KIO::CacheControl cachePolicy = dl ? dl->cachePolicy() : KIO::CC_Verify;
 
-    CachedObject* o = cache->find(kurl.url());
+    QString url = kurl.url();
+    CachedObject* o = cache->find(url);
 
     if ( o && o->type() != CachedType ) {
         removeCacheEntry( o );
         o = 0;
     }
 
-    if ( o && dl->needReload( kurl ) ) {
+    if ( o && dl->needReload( o, url ) ) {
         o = 0;
-        assert( cache->find( kurl.url() ) == 0 );
+        assert( cache->find( url ) == 0 );
     }
 
     if(!o)
@@ -1280,8 +1444,8 @@ CachedObjectType* Cache::requestObject( DocLoader* dl, const KURL& kurl, const c
 #ifdef CACHE_DEBUG
         kdDebug( 6060 ) << "Cache: new: " << kurl.url() << endl;
 #endif
-        CachedObjectType* cot = new CachedObjectType(dl, kurl.url(), cachePolicy, accept);
-        cache->insert( kurl.url(), cot );
+        CachedObjectType* cot = new CachedObjectType(dl, url, cachePolicy, accept);
+        cache->insert( url, cot );
         if ( cot->allowInLRUList() )
             insertInLRUList( cot );
         o = cot;
@@ -1332,9 +1496,12 @@ void Cache::flush(bool force)
 #endif
     }
 
-    for ( CachedObject* p = freeList->first(); p; p = freeList->next() ) {
+    for ( freeList->first(); freeList->current(); ) {
+        CachedObject* p = freeList->current();
         if ( p->canDelete() )
             freeList->remove();
+        else
+            freeList->next();
     }
 
 }
@@ -1503,7 +1670,7 @@ void Cache::insertInLRUList(CachedObject *object)
 // --------------------------------------
 
 void CachedObjectClient::setPixmap(const QPixmap &, const QRect&, CachedImage *) {}
-void CachedObjectClient::setStyleSheet(const DOM::DOMString &/*url*/, const DOM::DOMString &/*sheet*/) {}
+void CachedObjectClient::setStyleSheet(const DOM::DOMString &/*url*/, const DOM::DOMString &/*sheet*/, const DOM::DOMString &/*charset*/) {}
 void CachedObjectClient::notifyFinished(CachedObject * /*finishedObj*/) {}
 void CachedObjectClient::error(int /*err*/, const QString &/*text*/) {}
 
