@@ -371,12 +371,12 @@ RenderLayer* RenderObject::findNextLayer(RenderLayer* parentLayer, RenderObject*
     // Error check the parent layer passed in.  If it's null, we can't find anything.
     if (!parentLayer)
         return 0;
-    
+
     // Step 1: If our layer is a child of the desired parent, then return our layer.
     RenderLayer* ourLayer = layer();
     if (ourLayer && ourLayer->parent() == parentLayer)
         return ourLayer;
-    
+
     // Step 2: If we don't have a layer, or our layer is the desired parent, then descend
     // into our siblings trying to find the next layer whose parent is the desired parent.
     if (!ourLayer || ourLayer == parentLayer) {
@@ -387,17 +387,17 @@ RenderLayer* RenderObject::findNextLayer(RenderLayer* parentLayer, RenderObject*
                 return nextLayer;
         }
     }
-    
+
     // Step 3: If our layer is the desired parent layer, then we're finished.  We didn't
     // find anything.
     if (parentLayer == ourLayer)
         return 0;
-    
+
     // Step 4: If |checkParent| is set, climb up to our parent and check its siblings that
     // follow us to see if we can locate a layer.
     if (checkParent && parent())
         return parent()->findNextLayer(parentLayer, this, true);
-    
+
     return 0;
 }
 
@@ -474,7 +474,7 @@ int RenderObject::offsetTop() const
 
 RenderObject* RenderObject::offsetParent() const
 {
-    if (isBody()) 
+    if (isBody())
         return 0;
 
     // can't really use containing blocks here (#113280)
@@ -654,7 +654,7 @@ bool RenderObject::sizesToMaxWidth() const
 
     // Children of a horizontal marquee do not fill the container by default.
     // FIXME: Need to deal with MAUTO value properly.  It could be vertical.
-    if (parent()->style()->overflow() == OMARQUEE) {
+    if (parent()->style()->overflowX() == OMARQUEE) {
         EMarqueeDirection dir = parent()->style()->marqueeDirection();
         if (dir == MAUTO || dir == MFORWARD || dir == MBACKWARD || dir == MLEFT || dir == MRIGHT)
             return true;
@@ -1274,6 +1274,7 @@ void RenderObject::setStyle(RenderStyle *style)
         if ( d >= RenderStyle::Visible && !isText() && m_parent &&
              ( d == RenderStyle::Position ||
                m_style->outlineWidth() > style->outlineWidth() ||
+               (!m_style->hidesOverflow() && style->hidesOverflow()) ||
                ( m_style->hasClip() && !(m_style->clip() == style->clip()) ) ) ) {
             // schedule a repaint with the old style
             if (layer() && !isInlineFlow())
@@ -1394,7 +1395,7 @@ bool RenderObject::attemptDirectLayerTranslation()
         }
         if (needsDocSizeUpdate && canvas()) {
             bool posXOffset = (xPos()-oldXPos >= 0);
-            bool posYOffset = (yPos()-oldYPos >= 0);                         
+            bool posYOffset = (yPos()-oldYPos >= 0);
             canvas()->updateDocSizeAfterLayerTranslation(this, posXOffset, posYOffset);
         }
     }
@@ -1808,7 +1809,7 @@ void RenderObject::setInnerNode(NodeInfo& info)
 
     if (!info.innerNode() && element())
         info.setInnerNode(element());
-            
+
     if(!info.innerNonSharedNode() && element())
         info.setInnerNonSharedNode(element());
 }
@@ -1848,27 +1849,19 @@ short RenderObject::getVerticalPosition( bool firstLine, RenderObject* ref ) con
                 return vpos - style()->verticalAlignLength().width( lineHeight( firstLine ) );
 
             const QFont &f = ref->font( firstLine );
-            int fontheight = ref->lineHeight( firstLine );
             int fontsize = f.pixelSize();
-            int halfleading = ( fontheight - fontsize ) / 2;
 
             if ( va == SUB )
                 vpos += fontsize/5 + 1;
             else if ( va == SUPER )
                 vpos -= fontsize/3 + 1;
             else if ( va == TEXT_TOP ) {
-//                 qDebug( "got TEXT_TOP vertical pos hint" );
-//                 qDebug( "parent:" );
-//                 qDebug( "CSSLH: %d, CSS_FS: %d, basepos: %d", fontheight, fontsize, ref->baselinePosition( firstLine ) );
-//                 qDebug( "this:" );
-//                 qDebug( "CSSLH: %d, CSS_FS: %d, basepos: %d", lineHeight( firstLine ), style()->font().pixelSize(), baselinePosition( firstLine ) );
-                vpos += ( baselinePosition( firstLine ) - ref->baselinePosition( firstLine ) +
-                        halfleading );
+                vpos += baselinePosition( firstLine ) - (QFontMetrics(f).ascent() + QFontMetrics(f).leading()/2);
             } else if ( va == MIDDLE ) {
                 QRect b = QFontMetrics(f).boundingRect('x');
                 vpos += -b.height()/2 - lineHeight( firstLine )/2 + baselinePosition( firstLine );
             } else if ( va == TEXT_BOTTOM ) {
-                vpos += QFontMetrics(f).descent();
+                vpos += QFontMetrics(f).descent() + QFontMetrics(f).leading()/2;
                 if ( !isReplaced() )
                     vpos -= fontMetrics(firstLine).descent();
             } else if ( va == BASELINE_MIDDLE )
@@ -2097,6 +2090,9 @@ CounterNode* RenderObject::getCounter(const QString& counter, bool view, bool co
     if (style()->hasCounterReset(counter) || isRoot()) {
         i = new CounterReset(this);
         val = style()->counterReset(counter);
+        if (style()->hasCounterIncrement(counter)) {
+            val += style()->counterIncrement(counter);
+        }
 //         kdDebug( 6040 ) << renderName() << " counter-reset: " << counter << " " << val << endl;
     }
     else
