@@ -24,7 +24,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //----------------------------------------------------------------------------
 //
 // KDE File Manager -- HTTP Cookie Dialogs
-// $Id: kcookiewin.cpp,v 1.58 2005/02/15 21:27:16 waba Exp $
+// $Id: kcookiewin.cpp 395252 2005-03-06 14:46:36Z waba $
 
 // The purpose of the QT_NO_TOOLTIP and QT_NO_WHATSTHIS ifdefs is because
 // this file is also used in Konqueror/Embedded. One of the aims of
@@ -113,7 +113,18 @@ KCookieWin::KCookieWin( QWidget *parent, KHttpCookieList cookieList,
     QLabel* lbl = new QLabel( txt, vBox );
     lbl->setAlignment( Qt::AlignCenter );
     KHttpCookiePtr cookie = cookieList.first();
-    txt = i18n("<b>%1</b>").arg( KIDNA::toUnicode(cookie->host()) );
+
+    QString host (cookie->host());
+    int pos = host.find(':');
+    if ( pos > 0 )
+    {
+      QString portNum = host.left(pos);
+      host.remove(0, pos+1);
+      host += ':';
+      host += portNum;
+    }
+
+    txt = QString("<b>%1</b>").arg( KIDNA::toUnicode(host) );
     if (cookie->isCrossDomain())
        txt += i18n(" <b>[Cross Domain!]</b>");
     lbl = new QLabel( txt, vBox );
