@@ -1,5 +1,5 @@
 /*
- $Id: kthemebase.cpp,v 1.19 2003/09/06 19:06:37 binner Exp $
+ $Id: kthemebase.cpp 465272 2005-09-29 09:47:40Z mueller $
 
  This file is part of the KDE libraries
  Copyright (C) 1999 Daniel M. Duley <mosfet@kde.org>
@@ -35,8 +35,8 @@
 
  You should have received a copy of the GNU Library General Public License
  along with this library; see the file COPYING.LIB.  If not, write to
- the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- Boston, MA 02111-1307, USA.
+ the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ Boston, MA 02110-1301, USA.
 */
 
 #include "kthemebase.h"
